#include "game.h"
#include "player.h"

using namespace LIME;
 
Player::Player()
{
	addTypeID( "player" );

	addRootState( "root", new PlayerRootState );

	addState( "cpu", new PlayerCPUState( this ) );
	addState( "human", new PlayerHumanState( this ) );

	fall = false;
	knocked = 0;
	blocking = false;
}

void PlayerCPUState::enter()
{
	double radius = parseDouble( scene.getUserData()->getAttribute( "radius" ) );

	player->setPosition( radius / 2, 0, 0 );
	player->yaw( 90 );
}

void PlayerCPUState::exit()
{
}

void PlayerCPUState::update()
{
	Player* opponent = player->getName().compare( "alfred" ) ? ( Player* )scene.getObject( "alfred" ) : ( Player* )scene.getObject( "guthrum" );

	double x = ( opponent->getPosition() - player->getPosition() ).normalisedCopy().x;
	double y = ( opponent->getPosition() - player->getPosition() ).normalisedCopy().y;

	double xMove = x * ( ( opponent->getPosition() - player->getPosition() ).length() > 300 ) * timer.getDelta() * 120;
	double yMove = y * ( ( opponent->getPosition() - player->getPosition() ).length() > 300 ) * timer.getDelta() * 120;

	Ogre::AnimationState* idle, * run, * jump, * attack, * block;
	idle = player->entity->getAnimationState( "idle" );
	run = player->entity->getAnimationState( "run" );
	jump = player->entity->getAnimationState( "jump" );
	attack = player->entity->getAnimationState( "attack" );
	block = player->entity->getAnimationState( "block" );

	if( player->getPosition().length() > parseDouble( scene.getUserData()->getAttribute( "radius" ) ) )
	{
		player->translate( 0, 0, -timer.getDelta() * 600 );

		if( player->getPosition().z < -300 && player->getPosition().z >= -4200 ) 
		{
			if( !( ( Game* )core.getPerformance() )->ending )
			{
				( ( Game* )core.getPerformance() )->ending = true;

				FMOD::Channel* music;
				audio.system->getChannel( 0, &music );
				music->setLoopCount( 0 );
				audio.system->playSound( FMOD_CHANNEL_REUSE, audio.sounds[ "victory" ], 0, &music );
			}

			return;
		}

		if( player->getPosition().z < -4200 ) 
		{
			( ( Game* )core.getPerformance() )->end = true;
			( ( Game* )core.getPerformance() )->ending = false;
			return;
		}

		jump->setEnabled( true );
		run->setEnabled( false );
		jump->setLoop( false );

		return;
	}

	if( player->blocking )
	{
		block->setEnabled( true );
		block->setLoop( false );
		attack->setEnabled( false );
		idle->setEnabled( false );
		run->setEnabled( false );

		if( player->animSpeeds[ "block" ] == -2 && block->getTimePosition() == 0 ) 
		{
			player->blocking = false;
		    block->setEnabled( false );
		    idle->setEnabled( true );
		}

		return;
	}

	else
	{
		attack->addTime( timer.getDelta() );
		attack->setEnabled( false );
		idle->setEnabled( true );
	}

	static knockBack = 0;

	if( player->knocked )
	{
		run->setEnabled( true );
		attack->setEnabled( false );
		idle->setEnabled( false );
		block->setEnabled( false );
		player->animSpeeds[ "run" ] = -2;

		Vector3 movement = ( player->getPosition() - opponent->getPosition() ).normalisedCopy() * timer.getDelta() * 150;
		player->translate( movement, TS_WORLD );

		knockBack += movement.length();

		Quaternion q;
		q.FromAxes( Vector3( -movement.y, movement.x, 0 ), -movement, Vector3::UNIT_Z );
		q.normalise();

		player->setOrientation( q );

		if( knockBack > 100 ) 
		{
			player->knocked = 0;
			knockBack = 0;
			run->setEnabled( false );
			idle->setEnabled( true );
			player->animSpeeds[ "run" ] = 1;
		}

		return;
	}

	if( player->knocked == 1 )
	{
		block->setEnabled( true );
		attack->setEnabled( false );
		idle->setEnabled( false );
		run->setEnabled( false );

		Vector3 movement = ( player->getPosition() - opponent->getPosition() ).normalisedCopy() * timer.getDelta() * 150;
		player->translate( movement, TS_WORLD );

		knockBack += movement.length();

		Quaternion q;
		q.FromAxes( Vector3( -movement.y, movement.x, 0 ), -movement, Vector3::UNIT_Z );
		q.normalise();

		player->setOrientation( q );

		if( knockBack > 300 ) 
		{
			player->knocked = 0;
			knockBack = 0;
			block->setEnabled( false );
			idle->setEnabled( true );
		}

		return;
	}

	if( ( player->getPosition() + Vector3( xMove, yMove, 0 ) - opponent->getPosition() ).length() > 300 )
	{
		player->translate( xMove, yMove, 0, TS_WORLD );
	}

	Quaternion q;
	q.FromAxes( Vector3( y, -x, 0 ), Vector3( x, y, 0 ), Vector3::UNIT_Z );
	q.normalise();

	if( ( opponent->getPosition() - player->getPosition() ).length() > 301 ) 
	{
		player->setOrientation( q );
		run->setEnabled( true );
		player->animSpeeds[ "run" ] = 1.5;
		idle->setEnabled( false );
	}
	else 
	{
		run->setEnabled( false );
		idle->setEnabled( true );
	}
}

void PlayerHumanState::enter()
{
	double radius = parseDouble( scene.getUserData()->getAttribute( "radius" ) );

	player->setPosition( -radius / 2, 0, 0 );
	player->yaw( -90 );
}

void PlayerHumanState::exit()
{
}

void PlayerHumanState::update()
{
	double x = input.getJoystick()->getAxis( 0 )->getPositiveX()->getAnalog() - 
		       input.getJoystick()->getAxis( 0 )->getNegativeX()->getAnalog();
	double y = input.getJoystick()->getAxis( 0 )->getNegativeY()->getAnalog() - 
		       input.getJoystick()->getAxis( 0 )->getPositiveY()->getAnalog();

	double xMove = ( abs( x ) > 0.6 ? x / abs( x ) : 0 ) * timer.getDelta() * 270;
	double yMove = ( abs( y ) > 0.6 ? y / abs( y ) : 0 ) * timer.getDelta() * 270;

	Player* opponent = player->getName().compare( "alfred" ) ? ( Player* )scene.getObject( "alfred" ) : ( Player* )scene.getObject( "guthrum" );

	Ogre::AnimationState* idle, * run, * jump, * attack, * block;
	idle = player->entity->getAnimationState( "idle" );
	run = player->entity->getAnimationState( "run" );
	jump = player->entity->getAnimationState( "jump" );
	attack = player->entity->getAnimationState( "attack" );
	block = player->entity->getAnimationState( "block" );

	if( player->getPosition().length() > parseDouble( scene.getUserData()->getAttribute( "radius" ) ) )
	{
		player->translate( 0, 0, -timer.getDelta() * 600 );

		if( player->getPosition().z < -300 && player->getPosition().z >= -1800 ) 
		{
			if( !( ( Game* )core.getPerformance() )->ending )
			{
				( ( Game* )core.getPerformance() )->ending = true;

				FMOD::Channel* music;
				audio.system->getChannel( 0, &music );
				music->setLoopCount( 0 );
				audio.system->playSound( FMOD_CHANNEL_REUSE, audio.sounds[ "lose" ], 0, &music );
			}

			return;
		}

		if( player->getPosition().z < -1800 ) 
		{
			( ( Game* )core.getPerformance() )->end = true;
			( ( Game* )core.getPerformance() )->ending = false;
			return;
		}

		jump->setEnabled( true );
		run->setEnabled( false );
		jump->setLoop( false );

		return;
	}

	if( input.getJoystick()->getButton( 4 )->getFired() )
	{
		player->blocking = true;
		player->animSpeeds[ "block" ] = 2;
		return;
	}

	if( input.getJoystick()->getButton( 4 )->getCeased() )
	{
		player->animSpeeds[ "block" ] = -2;
		return;
	}

	if( player->blocking )
	{
		block->setEnabled( true );
		block->setLoop( false );
		attack->setEnabled( false );
		idle->setEnabled( false );
		run->setEnabled( false );

		if( player->animSpeeds[ "block" ] == -2 && block->getTimePosition() == 0 ) 
		{
			player->blocking = false;
		    block->setEnabled( false );
		    idle->setEnabled( true );
		}

		return;
	}

	if( input.getJoystick()->getButton( 0 )->getFired() )
	{
		FMOD::Channel* music;
		audio.system->getChannel( 6, &music );
		audio.system->playSound( FMOD_CHANNEL_REUSE, audio.sounds[ "sword" ], 0, &music );
	}
	if( input.getJoystick()->getButton( 0 )->getCeased() )
	{
		FMOD::Channel* music;
		audio.system->getChannel( 6, &music );
		music->stop();
	}

	if( input.getJoystick()->getButton( 0 )->getDigital() || 
		( attack->getEnabled() && attack->getTimePosition() < 0.6 ) )
	{
		attack->setEnabled( true );
		player->animSpeeds[ "attack" ] = 1.5;
		run->setEnabled( false );
		idle->setEnabled( false );

		if( player->knocked == 2 ) player->knocked = 1;

		if( ( player->getPosition() - opponent->getPosition() ).length() <= 200 )
		{
			if( ( opponent->blocking && 
				  abs( ( opponent->getOrientation().getYaw() - player->getOrientation().getYaw() ).valueAngleUnits() ) > 120 ) )
			{
				opponent->knocked = 1;

				if( input.getJoystick()->getButton( 0 )->getFired() ) 
				{
					audio.system->playSound( FMOD_CHANNEL_FREE, audio.sounds[ "sword" ], 0, 0 );
				}
			}
			else opponent->knocked = 2;
		}
		
		return;
	}
	else
	{
		attack->setTimePosition( 0 );
		attack->setEnabled( false );
		idle->setEnabled( true );
	}

	if( ( player->getPosition() + Vector3( xMove, yMove, 0 ) - opponent->getPosition() ).length() > 150 )
	{
		player->translate( xMove, yMove, 0, TS_WORLD );
		( ( Camera* )scene.getObject( "freecam" ) )->translate( xMove, yMove, 0, TS_WORLD );
	}

	Quaternion q;
	q.FromAxes( Vector3( y, -x, 0 ), Vector3( x, y, 0 ), Vector3::UNIT_Z );
	q.normalise();

	if( abs( x ) > 0.6 || abs( y ) > 0.6 ) 
	{
		player->setOrientation( q );
		run->setEnabled( true );
		player->animSpeeds[ "run" ] = 2.5;
		idle->setEnabled( false );
	}
	else 
	{
		run->setEnabled( false );
		idle->setEnabled( true );
	}
}